#!/usr/bin/env python

"""Entry point for AUIMD.

If this module is run as the main module, it will :
- chdir to this module path
- analyse command line arguments
- create a Desktop object
- start a QApplication with the desktop widget as main window

"""

import os
import sys
import socket

from PyQt4 import QtCore, QtGui

# make sure we interpret paths relative to the location of the auimd.py entry point
os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])))

from core.desktop import *

def usage():
  print "AUIMD - Aggregated User Interface for Mobile Devices.  Usage:"
  print
  print "    "+sys.argv[0]+" [options]"
  print
  print "Options:"
  print
  print "  -device <device>             Set the hardware type (default='dummy',"
  print "                               show available='list')."
  print "  -fullscreen                  Use the whole screen and a frameless "
  print "                               window (default=no)."
  print "  -no-cursor                   Hide the cursor (default=no)."
  print "  -embed <win-id>              Embed into an existing window."
  print "  -help                        Print this message."
  print
  print "Some other Qt/X11 usefull options:"
  print
  print "  -display <display>            X11 display to use, default : $DISPLAY"
  print "  -geometry <WxH[+X+Y]>         Window geometry, overriden by -fullscreen"
  print

  sys.exit(1)

if __name__=="__main__":
  # decode arguments
  fullscreen=0
  no_cursor=0
  device="dummy"
  embed=None
  for i in range(len(sys.argv)):
    if sys.argv[i]=="-help":
      usage()
    elif sys.argv[i]=="-fullscreen":
      fullscreen=1
    elif sys.argv[i]=="-no-cursor":
      no_cursor=1
    elif sys.argv[i]=="-embed":
      if i==len(sys.argv)-1: usage()
      i=i+1
      embed=sys.argv[i]
    elif sys.argv[i]=="-device":
      if i==len(sys.argv)-1: usage()
      i=i+1
      device=sys.argv[i]
      if device=="list":
        print "Available devices : "
        for i in os.listdir("devices"):
          if os.path.exists("devices/"+i+"/device.py"): print i
        sys.exit(1)
    elif sys.argv[i]=="-display":
      if i==len(sys.argv)-1: usage()
      # propagate the display value so that other processes inherit from it
      i=i+1
      os.environ["DISPLAY"]=sys.argv[i]

  app=QtGui.QApplication(sys.argv)
  dbus.mainloop.qt.DBusQtMainLoop(set_as_default=True)
  dbus.SessionBus().request_name("net.pierrox.auimd")
  d=createDesktop(device, fullscreen, no_cursor, embed)
  d.show()
  app.exec_()
