from PyQt4 import QtCore, QtGui

from core.pewidget import *

class FileOpenSmall(QtGui.QWidget, PeWidget):
  @staticmethod
  def defaultWidgetIcon():
    return QtGui.QIcon("core/icons/fileopen.png")

  @staticmethod
  def defaultWidgetTitle():
    return "Open..."

  def __init__(self, parent):
    QtGui.QWidget.__init__(self, parent)
    PeWidget.__init__(self, parent)

    self.path=QtGui.QComboBox(self)
    self.sort=QtGui.QComboBox(self)
    self.files=QtGui.QListWidget(self)

    self.sort.addItem("Name")
    self.sort.addItem("Time")
    self.sort.addItem("Size")
    Self.sort.addItem("Type")

    hlayout=QtGui.QHBoxLayout()
    hlayout.addWidget(self.path)
    hlayout.addWidget(self.sort)

    vlayout=QtGui.QVBoxLayout(self)
    vlayout.addLayout(hlayout)
    vlayout.addWidget(self.files)

    self.folder_icon=QtGui.QIcon("core/icons/folder.png")
    self.file_icon=QtGui.QIcon("core/icons/empty.png")

    self.loop=None
    self.selected_file=None

    self.left_button.setText("OK")
    self.right_button.setText("Cancel")

    QtCore.QObject.connect(self.files, QtCore.SIGNAL("itemClicked(QListWidgetItem*)"), self.fileSelected)
    QtCore.QObject.connect(self.files, QtCore.SIGNAL("itemActivated(QListWidgetItem*)"), self.fileActivated)

  def openFile(self, path=None, filter=None):
    if(path==None):
      d=QtCore.QDir.current()
    else:
      d=QtCore.QDir(path)
      if(not d.exists() or not d.isReadable()):
        d=QtCore.QDir.current()

    self.changeDirectory(d)
    self.loop=QtCore.QEventLoop(self)
    self.loop.exec_()

    return self.selected_file

  def leftButtonAction(self):
    self.loop.exit()

  def rightButtonAction(self):
    self.selected_file=None
    self.loop.exit()

  def changeDirectory(self, new_directory):
    QtCore.QObject.disconnect(self.path, QtCore.SIGNAL("currentIndexChanged(int)"), self.directorySelected)

    self.selected_file=None

    self.current_path=new_directory
    d=QtCore.QDir(self.current_path.absolutePath())
    self.path.clear()
    i=""
    s=d.dirName()
    self.nb_path_elem=0
    while(s):
      self.path.insertItem(0, s)
      d.cdUp()
      s=d.dirName()
      self.nb_path_elem+=1
    self.path.insertItem(0, "/")

    self.files.clear()
    files_info=self.current_path.entryInfoList(["*"], QtCore.QDir.AllEntries, QtCore.QDir.DirsFirst)
    for fi in files_info:
      if(fi.fileName()=="." or (fi.fileName()==".." and new_directory=="/")):
        continue
      if(fi.isDir()):
        QtGui.QListWidgetItem(self.folder_icon, fi.fileName(), self.files)
      else:
        QtGui.QListWidgetItem(self.file_icon, fi.fileName(), self.files)

    QtCore.QObject.connect(self.path, QtCore.SIGNAL("currentIndexChanged(int)"), self.directorySelected)
    self.left_button.setEnabled(0)

  def directorySelected(self, i):
    d=QtCore.QDir(self.current_path.absolutePath())
    for e in range(self.nb_path_elem-i):
      d.cdUp()

    self.changeDirectory(d)

  def fileSelected(self, f, activate=0):
    filepath=self.current_path.filePath(f.text())
    fi=QtCore.QFileInfo(filepath)
    if(fi.isDir()):
      self.changeDirectory(QtCore.QDir(filepath))
    else:
      self.selected_file=filepath
      if(activate):
        self.loop.exit()
      else:
        self.left_button.setEnabled(1)

  def fileActivated(self, f):
    self.fileSelected(f, 1)
        
  def closeEvent(self, e):
    if(self.loop): 
      self.selected_file=None
      self.loop.exit()
