import random
from PyQt4 import QtCore, QtGui

from . import __path__
from core.auimd_widget import *

icons_path=__path__[0]+"/icons"

# add 2 on the width, and 1 on the height for margins
board_width=10
board_height=20

class Tetris(QtGui.QWidget, AuimdWidget):

  @staticmethod
  def defaultWidgetIcon():
    return QtGui.QIcon(icons_path+"/fifteenpieces.png")

  @staticmethod
  def defaultWidgetTitle():
    return "Tetris"

  @staticmethod
  def getCategory():
    return "Games"

  def __init__(self, parent):
    QtGui.QWidget.__init__(self, parent)
    AuimdWidget.__init__(self, parent)

    self.game=Game(self)
    self.game.setFixedSize(10*(board_width+2), 10*(board_height+1))

    self.bt_start_pause=QtGui.QPushButton(QtGui.QIcon(icons_path+"/player_play.png"), "", self)
    self.bt_stop=QtGui.QPushButton(QtGui.QIcon(icons_path+"/player_stop.png"), "", self)
 
    self.status=QtGui.QLabel("stopped", self)
    self.status.setAlignment(QtCore.Qt.AlignHCenter)

    self.lb1=QtGui.QLabel("Level : ", self)
    self.level=QtGui.QLabel("0", self)
    self.level.setFrameStyle(QtGui.QFrame.Sunken | QtGui.QFrame.Panel)
    self.lb2=QtGui.QLabel("Score : ", self)
    self.score=QtGui.QLabel("0", self)
    self.score.setFrameStyle(QtGui.QFrame.Sunken | QtGui.QFrame.Panel)

    self.bt_left=QtGui.QPushButton(QtGui.QIcon(icons_path+"/1leftarrow.png"), "", self)
    self.bt_right=QtGui.QPushButton(QtGui.QIcon(icons_path+"/1rightarrow.png"), "", self)

    self.bt_rotate_ccw=QtGui.QPushButton(QtGui.QIcon(icons_path+"/rotate.png"), "", self)
    self.bt_rotate_cw=QtGui.QPushButton(QtGui.QIcon(icons_path+"/rotate_cw.png"), "", self)

    self.bt_down=QtGui.QPushButton(QtGui.QIcon(icons_path+"/bottom.png"), "", self)

    for b in [self.bt_start_pause, self.bt_stop, self.bt_left, self.bt_right, self.bt_rotate_ccw, self.bt_rotate_cw, self.bt_down]:
      b.setFocusPolicy(QtCore.Qt.NoFocus)
      b.setEnabled(0)

    self.bt_start_pause.setEnabled(1)

    QtCore.QObject.connect(self.bt_start_pause, QtCore.SIGNAL("clicked()"), self.doStartPause)
    QtCore.QObject.connect(self.bt_stop, QtCore.SIGNAL("clicked()"), self.doStop)
    QtCore.QObject.connect(self.bt_left, QtCore.SIGNAL("clicked()"), self.game.moveLeft)
    QtCore.QObject.connect(self.bt_right, QtCore.SIGNAL("clicked()"), self.game.moveRight)
    QtCore.QObject.connect(self.bt_down, QtCore.SIGNAL("clicked()"), self.game.moveDown)
    QtCore.QObject.connect(self.bt_rotate_ccw, QtCore.SIGNAL("clicked()"), self.game.rotateCCW)
    QtCore.QObject.connect(self.bt_rotate_cw, QtCore.SIGNAL("clicked()"), self.game.rotateCW)
    QtCore.QObject.connect(self.game, QtCore.SIGNAL("started()"), self.gameStarted)
    QtCore.QObject.connect(self.game, QtCore.SIGNAL("stopped()"), self.gameStopped)
    QtCore.QObject.connect(self.game, QtCore.SIGNAL("paused()"), self.gamePaused)
    QtCore.QObject.connect(self.game, QtCore.SIGNAL("scoreChanged(int,int)"), self.scoreChanged)
    
    layout_bt_1=QtGui.QHBoxLayout()
    layout_bt_1.addWidget(self.bt_start_pause)
    layout_bt_1.addWidget(self.bt_stop)

    layout_labels=QtGui.QGridLayout()
    layout_labels.addWidget(self.lb1, 0, 0)
    layout_labels.addWidget(self.level, 0, 1)
    layout_labels.addWidget(self.lb2, 1, 0)
    layout_labels.addWidget(self.score, 1, 1)
     
    layout_bt_2=QtGui.QHBoxLayout()
    layout_bt_2.addWidget(self.bt_left)
    layout_bt_2.addWidget(self.bt_right)

    layout_bt_3=QtGui.QHBoxLayout()
    layout_bt_3.addWidget(self.bt_rotate_ccw)
    layout_bt_3.addWidget(self.bt_rotate_cw)

    layout_bt_4=QtGui.QHBoxLayout()
    layout_bt_4.addWidget(self.bt_down)

    layout_left=QtGui.QVBoxLayout()
    layout_left.addLayout(layout_bt_1)
    layout_left.addStretch()
    layout_left.addWidget(self.status)
    layout_left.addLayout(layout_labels)
    layout_left.addStretch()
    layout_left.addLayout(layout_bt_2)
    layout_left.addLayout(layout_bt_3)
    layout_left.addLayout(layout_bt_4)

    layout_right=QtGui.QVBoxLayout()
    #layout_right.addStretch()
    layout_right.addWidget(self.game)

    layout=QtGui.QHBoxLayout(self)
    layout.addLayout(layout_right)
    layout.addLayout(layout_left)

    self.left_button.setText("Start !")
    self.right_button.setText("Close")

  def scoreChanged(self, score, level):
    self.score.setText(str(score))
    self.level.setText(str(level))

  def rightButtonAction(self):
    self.desktop.closeWidget(self)

  def leftButtonAction(self):
    self.doStartPause()

  def doStartPause(self):
    if self.game.started:
      if self.game.paused:
        self.game.start()
      else:
        self.game.pause()
    else:
      self.game.start()

  def doStop(self):
    self.game.stop()

  def setCommandButtonEnabled(self, e):
    for b in [self.bt_left, self.bt_right, self.bt_rotate_ccw, self.bt_rotate_cw, self.bt_down]:
      b.setEnabled(e)

  def gameStarted(self):
    self.bt_start_pause.setIcon(QtGui.QIcon(icons_path+"/player_pause.png"))
    self.left_button.setText("Pause")
    self.bt_stop.setEnabled(1)
    self.setCommandButtonEnabled(1)
    self.status.setText("playing...")

  def gameStopped(self):
    self.bt_start_pause.setIcon(QtGui.QIcon(icons_path+"/player_play.png"))
    self.left_button.setText("Start !")
    self.bt_stop.setEnabled(0)
    self.setCommandButtonEnabled(0)
    self.status.setText("stopped")

  def gamePaused(self):
    self.bt_start_pause.setIcon(QtGui.QIcon(icons_path+"/player_play.png"))
    self.left_button.setText("Resume")
    self.setCommandButtonEnabled(0)
    self.status.setText("paused")
  
class Game(QtGui.QWidget):
  def __init__(self, parent):
    QtGui.QWidget.__init__(self, parent)

    self.started=0
    self.paused=0
    self.current_block=None

    self.table=[[1 for row in range(board_width+2)] for row in range(board_height+1)]

    self.timer=QtCore.QTimer(self)
    QtCore.QObject.connect(self.timer, QtCore.SIGNAL("timeout()"), self.timeToMove)

    self.prepareBlocks()

  def initGame(self):
    self.started=0
    self.paused=0
    self.current_block=random.randint(0, len(self.blocks)-1)
    self.current_block_x=board_width/2-1
    self.current_block_y=0
    self.current_block_rotate=0
    self.next_block=0
    self.level=1
    self.score=0
    self.num_lines=0
    self.emit(QtCore.SIGNAL("scoreChanged(int,int)"), self.score, self.level)

    self.table=[[0 for row in range(board_width+2)] for col in range(board_height+1)]
    for y in range(board_height+1):
      self.table[y][0]=1
      self.table[y][board_width+1]=1
    for x in range(board_width+2):
      self.table[board_height][x]=1

    self.drawBackground()
  
  def resizeEvent(self, e):
    self.pw=self.width()/(board_width+2)
    self.ph=self.height()/(board_height+1)
    self.preparePieces()
    self.drawBackground()
     
  def drawBackground(self, highlight=0): 
    nb_lines=0
    self.background=QtGui.QImage(self.width(), self.height(), QtGui.QImage.Format_ARGB32)
    p=QtGui.QPainter(self.background)
    p.eraseRect(0, 0, self.background.width(), self.background.height())
    for y in range(board_height+1):
      n=0
      for x in range(board_width+2):
        if self.table[y][x]>0: n+=1
      if n==board_width+2 and y<board_height:
        nb_lines+=1
        if highlight:
          p.fillRect(self.pw, y*self.ph, self.pw*10, self.ph, QtGui.QColor(128, 128, 128))
          p.setCompositionMode(QtGui.QPainter.CompositionMode_Multiply)
      for x in range(board_width+2):
        p.drawImage(x*self.pw, y*self.ph, self.pieces[self.table[y][x]])
      if n==board_width+2 and y<board_height:
        p.setCompositionMode(QtGui.QPainter.CompositionMode_SourceOver)

    return nb_lines

  def paintEvent(self, e):
    p=QtGui.QPainter(self)

    if self.paused:
      p.setPen(QtCore.Qt.NoPen)
      gradient=QtGui.QLinearGradient(0, 0, self.width(), self.height())
      gradient.setColorAt(0, QtCore.Qt.black);
      gradient.setColorAt(0.5, QtCore.Qt.white);
      gradient.setColorAt(1, QtCore.Qt.black);
      p.setBrush(gradient);
      p.drawRect(1, 0, self.width(), self.height())
    else:
      p.drawImage(0, 0, self.background)

      if self.current_block!=None:
        b=self.blocks[self.current_block][self.current_block_rotate]
        for x in range(len(b)):
          col=b[x]
          for y in range(len(col)):
            v=self.pieces[col[y]]
            if col[y]:
              p.drawImage((self.current_block_x+x)*self.pw, (self.current_block_y+y)*self.ph, self.pieces[col[y]])

  def preparePieces(self):
    self.pieces=[]
    
    for i in range(len(self.blocks)+1):
      img=QtGui.QImage(self.pw, self.ph, QtGui.QImage.Format_RGB32)
      painter=QtGui.QPainter(img)
      painter.eraseRect(0, 0, self.pw, self.ph)
      if i==1:
        painter.setBrush(QtCore.Qt.yellow)
      elif i==2:
        painter.setBrush(QtCore.Qt.blue)
      elif i==3:
        painter.setBrush(QtCore.Qt.red)
      elif i==4:
        painter.setBrush(QtCore.Qt.green)
      elif i==5:
        painter.setBrush(QtCore.Qt.gray)
      elif i==6:
        painter.setBrush(QtCore.Qt.magenta)
      elif i==7:
        painter.setBrush(QtCore.Qt.cyan)
      elif i==8:
        painter.setBrush(QtGui.QColor(128, 128, 128))
      
      if i>0:
        painter.drawRect(0, 0, self.pw-1, self.ph-1)

      self.pieces.append(img)

  def prepareBlocks(self):
    self.blocks=[]

    bl_square=[ 
                [[0,0,0,0], [0,1,1,0], [0,1,1,0], [0,0,0,0]],
                [[0,0,0,0], [0,1,1,0], [0,1,1,0], [0,0,0,0]],
                [[0,0,0,0], [0,1,1,0], [0,1,1,0], [0,0,0,0]],
                [[0,0,0,0], [0,1,1,0], [0,1,1,0], [0,0,0,0]],
              ]

    bl_line=[ 
                [[0,0,0,0], [2,2,2,2], [0,0,0,0], [0,0,0,0]],
                [[0,2,0,0], [0,2,0,0], [0,2,0,0], [0,2,0,0]],
                [[0,0,0,0], [2,2,2,2], [0,0,0,0], [0,0,0,0]],
                [[0,2,0,0], [0,2,0,0], [0,2,0,0], [0,2,0,0]],
            ]

    bl_s=[ 
                [[0,0,0,0], [0,3,3,0], [0,0,3,3], [0,0,0,0]],
                [[0,0,3,0], [0,3,3,0], [0,3,0,0], [0,0,0,0]],
                [[0,0,0,0], [0,3,3,0], [0,0,3,3], [0,0,0,0]],
                [[0,0,3,0], [0,3,3,0], [0,3,0,0], [0,0,0,0]],
         ]

    bl_z=[ 
                [[0,0,0,0], [0,4,4,0], [4,4,0,0], [0,0,0,0]],
                [[4,0,0,0], [4,4,0,0], [0,4,0,0], [0,0,0,0]],
                [[0,0,0,0], [0,4,4,0], [4,4,0,0], [0,0,0,0]],
                [[4,0,0,0], [4,4,0,0], [0,4,0,0], [0,0,0,0]],
         ]

    bl_l_inv=[ 
                [[0,0,5,0], [5,5,5,0], [0,0,0,0], [0,0,0,0]],
                [[0,5,0,0], [0,5,0,0], [0,5,5,0], [0,0,0,0]],
                [[0,0,0,0], [5,5,5,0], [5,0,0,0], [0,0,0,0]],
                [[5,5,0,0], [0,5,0,0], [0,5,0,0], [0,0,0,0]],
         ]

    bl_l=[ 
                [[0,0,0,0], [6,6,6,0], [0,0,6,0], [0,0,0,0]],
                [[0,6,0,0], [0,6,0,0], [6,6,0,0], [0,0,0,0]],
                [[6,0,0,0], [6,6,6,0], [0,0,0,0], [0,0,0,0]],
                [[0,6,6,0], [0,6,0,0], [0,6,0,0], [0,0,0,0]],
             ]

    bl_triangle=[ 
                [[0,7,0,0], [7,7,7,0], [0,0,0,0], [0,0,0,0]],
                [[0,7,0,0], [7,7,0,0], [0,7,0,0], [0,0,0,0]],
                [[0,0,0,0], [7,7,7,0], [0,7,0,0], [0,0,0,0]],
                [[0,7,0,0], [0,7,7,0], [0,7,0,0], [0,0,0,0]],
                ]

    self.blocks=[ bl_square, bl_line, bl_s, bl_z, bl_l, bl_l_inv, bl_triangle ]
    
  def start(self):
    if not self.paused:
      self.initGame()

    self.started=1
    self.paused=0
    self.emit(QtCore.SIGNAL('started()'))

    self.timer.start(1000)

    self.update()

  def pause(self):
    self.paused=1
    self.timer.stop()
    self.emit(QtCore.SIGNAL('paused()'))
    self.update()

  def stop(self):
    self.timer.stop()
    self.paused=0
    self.started=0
    self.emit(QtCore.SIGNAL('stopped()'))
    self.update()

  def moveBlock(self, dx, dy, dr):
    r=self.current_block_rotate+dr
    if r>3: r=0
    elif r<0: r=3
    b=self.blocks[self.current_block][r]
    for x in range(len(b)):
      col=b[x]
      for y in range(len(col)):
        nx=self.current_block_x+x+dx
        ny=self.current_block_y+y+dy
        if nx>=0 and nx<board_width+2 and ny>0 and ny<board_height+1:
          if col[y]>0 and self.table[ny][nx]:
            return 0
    
    self.current_block_x+=dx
    self.current_block_y+=dy
    self.current_block_rotate=r
    return 1

  def moveLeft(self):
    self.moveBlock(-1, 0, 0)
    self.update()
    
  def moveRight(self):
    self.moveBlock(1, 0, 0)
    self.update()
    
  def moveDown(self):
    while self.moveBlock(0, 1, 0):
      self.score+=1
    self.emit(QtCore.SIGNAL("scoreChanged(int,int)"), self.score, self.level)
    self.fixBlock()
  
  def rotateCW(self):
    self.moveBlock(0, 0, -1)
    self.update()

  def rotateCCW(self):
    self.moveBlock(0, 0, 1)
    self.update()
   
  def timeToMove(self):
    if not self.moveBlock(0, 1, 0):
      self.fixBlock()
    else:
      self.update()

  def fixBlock(self):
    if self.current_block_y==0: self.stop()

    b=self.blocks[self.current_block][self.current_block_rotate]
    for x in range(len(b)):
      col=b[x]
      for y in range(len(col)):
        nx=self.current_block_x+x
        ny=self.current_block_y+y
        if nx>=0 and nx<board_width+2 and ny>0 and ny<board_height+1:
          if col[y]: self.table[ny][nx]=col[y]
    n=self.drawBackground(1)

    if n>0:
      self.score+=(0,100,200,400,800)[n]*self.level
      self.num_lines+=n
      self.level=1+self.num_lines/10
      if self.level>9: self.level=9
      self.emit(QtCore.SIGNAL("scoreChanged(int,int)"), self.score, self.level)
      self.timer.stop()
      self.timer.start(1000-(self.level-1)*100)
      for i in range(5):
        self.drawBackground(i&1)
        self.repaint()
        QtCore.QThread.currentThread().msleep(70)
 
      new_table=[0 for row in range(board_height+2)]
      ny=board_height
      for y in range(board_height, -1, -1):
        n=0
        for x in range(board_width+2):
          if self.table[y][x]>0: n+=1
        if n!=board_width+2 or y==board_height:
          new_table[ny]=self.table[y]
          ny-=1
        else:
          new_table[ny-y]=[0 for row in range(board_width+2)]
          new_table[ny-y][0]=random.randint(1, len(self.blocks)-1)
          new_table[ny-y][board_width+1]=new_table[ny-y][0]
      self.table=new_table
      self.drawBackground()
        
 
  
    self.current_block=random.randint(0, len(self.blocks)-1)
    self.current_block_x=board_width/2-1
    self.current_block_y=0
    self.current_block_rotate=0

    self.update()
